<?php

namespace Drupal\Tests\commerce_sermepa\FunctionalJavascript;

use Drupal\Tests\commerce\FunctionalJavascript\CommerceWebDriverTestBase;

/**
 * Tests the payment gateway UI for 'Sermepa' case.
 *
 * @group commerce_sermepa
 */
class SermepaGatewayUiTest extends CommerceWebDriverTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'commerce_sermepa',
  ];

  /**
   * {@inheritdoc}
   */
  protected function getAdministratorPermissions() {
    return array_merge([
      'administer commerce_payment_gateway',
    ], parent::getAdministratorPermissions());
  }

  /**
   * Tests creating a Sermepa payment gateway.
   */
  public function testPaymentGatewayCreation() {
    $this->drupalGet('admin/commerce/config/payment-gateways');
    $this->getSession()->getPage()->clickLink('Add payment gateway');
    $this->assertSession()->addressEquals('admin/commerce/config/payment-gateways/add');

    $radio_button = $this->getSession()->getPage()->findField('Sermepa/Redsýs');
    $radio_button->click();
    $this->waitForAjaxToFinish();

    $values = [
      'label' => 'Sermepa',
      'plugin' => 'commerce_sermepa',
      'configuration[commerce_sermepa][mode]' => 'test',
      'configuration[commerce_sermepa][merchant_name]' => 'Merchant name',
      'configuration[commerce_sermepa][merchant_code]' => '000000001',
      'configuration[commerce_sermepa][merchant_group]' => 'Merchant group',
      'configuration[commerce_sermepa][merchant_password]' => '00000000000000000000000000000000',
      'configuration[commerce_sermepa][merchant_terminal]' => '001',
      'configuration[commerce_sermepa][merchant_paymethods]' => ['C', 'D'],
      'configuration[commerce_sermepa][merchant_consumer_language]' => '001',
      'configuration[commerce_sermepa][currency]' => 840,
      'configuration[commerce_sermepa][transaction_type]' => 0,
      'configuration[commerce_sermepa][instructions][value]' => 'Instructions',
      'configuration[commerce_sermepa][instructions][format]' => 'plain_text',
      'status' => '1',
      // Setting the 'id' can fail if focus switches to another field.
      // This is a bug in the machine name JS that can be reproduced manually.
      'id' => 'sermepa',
    ];
    $this->submitForm($values, 'Save');
    $this->assertSession()->addressEquals('admin/commerce/config/payment-gateways');
    $this->assertSession()->responseContains('Sermepa');
    $this->assertSession()->responseContains('Test');

    $payment_gateway = $this->container->get('entity_type.manager')->getStorage('commerce_payment_gateway')->load('sermepa');
    $this->assertEquals('sermepa', $payment_gateway->id());
    $this->assertEquals('Sermepa', $payment_gateway->label());
    $this->assertEquals('commerce_sermepa', $payment_gateway->getPluginId());
    $payment_gateway_plugin = $payment_gateway->getPlugin();
    $this->assertEquals('test', $payment_gateway_plugin->getMode());
    $configuration = $payment_gateway_plugin->getConfiguration();
    $this->assertEquals('Merchant name', $configuration['merchant_name']);
    $this->assertEquals('000000001', $configuration['merchant_code']);
    $this->assertEquals('Merchant group', $configuration['merchant_group']);
    $this->assertEquals('00000000000000000000000000000000', $configuration['merchant_password']);
    $this->assertEquals('001', $configuration['merchant_terminal']);
    $this->assertEquals(['C', 'D'], $configuration['merchant_paymethods']);
    $this->assertEquals('001', $configuration['merchant_consumer_language']);
    $this->assertEquals(840, $configuration['currency']);
    $this->assertEquals(0, $configuration['transaction_type']);
    $this->assertEquals('Instructions', $configuration['instructions']['value']);
    $this->assertEquals('plain_text', $configuration['instructions']['format']);
  }

  /**
   * Tests editing a payment gateway.
   */
  public function testPaymentGatewayEditing() {
    $values = [
      'id' => 'edit_sermepa',
      'label' => 'Edit sermepa',
      'plugin' => 'commerce_sermepa',
      'status' => 0,
    ];
    $payment_gateway = $this->createEntity('commerce_payment_gateway', $values);

    $this->drupalGet('admin/commerce/config/payment-gateways/manage/' . $payment_gateway->id());
    $values += [
      'configuration[commerce_sermepa][mode]' => 'test',
      'configuration[commerce_sermepa][merchant_name]' => 'Merchant name',
      'configuration[commerce_sermepa][merchant_code]' => '000000001',
      'configuration[commerce_sermepa][merchant_group]' => 'Merchant group',
      'configuration[commerce_sermepa][merchant_password]' => '00000000000000000000000000000000',
      'configuration[commerce_sermepa][merchant_terminal]' => '001',
      'configuration[commerce_sermepa][merchant_paymethods]' => ['C', 'D'],
      'configuration[commerce_sermepa][merchant_consumer_language]' => '001',
      'configuration[commerce_sermepa][currency]' => 840,
      'configuration[commerce_sermepa][transaction_type]' => 0,
      'configuration[commerce_sermepa][instructions][value]' => 'Instructions',
      'configuration[commerce_sermepa][instructions][format]' => 'plain_text',
    ];
    $this->submitForm($values, 'Save');

    $this->container->get('entity_type.manager')->getStorage('commerce_payment_gateway')->resetCache();
    $payment_gateway = $this->container->get('entity_type.manager')->getStorage('commerce_payment_gateway')->load('edit_sermepa');
    $this->assertEquals('edit_sermepa', $payment_gateway->id());
    $this->assertEquals('Edit sermepa', $payment_gateway->label());
    $this->assertEquals('commerce_sermepa', $payment_gateway->getPluginId());
    $payment_gateway_plugin = $payment_gateway->getPlugin();
    $this->assertEquals('test', $payment_gateway_plugin->getMode());
    $configuration = $payment_gateway_plugin->getConfiguration();
    $this->assertEquals('Merchant name', $configuration['merchant_name']);
    $this->assertEquals('000000001', $configuration['merchant_code']);
    $this->assertEquals('Merchant group', $configuration['merchant_group']);
    $this->assertEquals('00000000000000000000000000000000', $configuration['merchant_password']);
    $this->assertEquals('001', $configuration['merchant_terminal']);
    $this->assertEquals(['C', 'D'], $configuration['merchant_paymethods']);
    $this->assertEquals('001', $configuration['merchant_consumer_language']);
    $this->assertEquals(840, $configuration['currency']);
    $this->assertEquals(0, $configuration['transaction_type']);
    $this->assertEquals('Instructions', $configuration['instructions']['value']);
    $this->assertEquals('plain_text', $configuration['instructions']['format']);
  }

}
